/*
    i2c.c - part of s_lm75
    
    Thomas Fischl <tfischl@gmx.de> 

    Uses "PCI Library" by Martin Mares.    

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Last change: 09/22/2004
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "i2c.h"
#include "lib/pci.h"

struct pci_access *pacc;
struct pci_dev *pcidev;


void out(int scl, int sda) {
	byte value;
	value = pci_read_byte(pcidev, 0x91);
	value &= ~(1<<PIN_SCL);
	value |= scl << PIN_SCL;
	value &= ~(1<<PIN_SDA);
	value |= sda << PIN_SDA;
	pci_write_byte(pcidev, 0x91, value);
}


int read_pin(int pin) {
	byte value;
        value = pci_read_byte(pcidev, 0x91);
	return ((value >> pin) & 0x01);
}


void set_output(int pin) {
	byte value;	
	value = pci_read_byte(pcidev, 0x90);
	value |= (1<<pin);
	pci_write_byte(pcidev, 0x90, value);
}


void set_input(int pin) {
	byte value;	
	value = pci_read_byte(pcidev, 0x90);
	value &= ~(1<<pin);
	pci_write_byte(pcidev, 0x90, value);
}


void i2c_delay(void) {
//	usleep(1);
}


void i2c_init() {
	unsigned int c;

	pacc = pci_alloc();

	pci_init(pacc);
    	pcidev = pci_get_dev(pacc, 0x00, 0x12, 0x00);

        c = pci_read_word(pcidev, 0x00);
	if (c != 0x1078) {
		fprintf(stderr, "error: CS5530 not found!\n");
		exit(-1);
	}

	/* SCL und SDA Eingaenge und auf HIGH */
	set_output(PIN_SDA);
	set_output(PIN_SCL);
	out(1, 1);
	i2c_delay();
}


void i2c_cleanup() {
	out(1, 1);

	pci_free_dev(pcidev);
	pci_cleanup(pacc);
}


void i2c_start() {
	/* SDA wird heruntergezogen, waehrend SCL high bleibt */
	out(1, 1);
	i2c_delay();
	out(1, 0);
	i2c_delay();

	/* Clockleitung zum Senden vorbereiten */
	out(0, 0);
	i2c_delay();
}


void i2c_stop() {

	i2c_delay();
	/* SDA wechselt von low nach high, waehrend SCL high bleibt */
	out(1, 0);
	i2c_delay();

	out(1, 1);
}


int i2c_sendbyte(int value) {
	int i, ret=0, sda;

	for(i = 7; i > -1; i--) {
		sda = ((value >> i) & 0x01);
		out(0, sda);
		/* Takten */
		i2c_delay();
		out(1, sda);
		i2c_delay();
		out(0, 0);
	}

	set_input(PIN_SDA);
	i2c_delay();
	out(1, 0);
	i2c_delay();
	ret = read_pin(PIN_SDA);
	set_output(PIN_SDA);
	out(0, 0);

	return !ret;
}


int i2c_readbyte(int ack_control) {
	int i, val = 0;

	out(0, 1);
	set_input(PIN_SDA);
	
	for(i = 7; i > -1; i--) {
		i2c_delay();
		out(1, 1);
		i2c_delay();		
		val |= (read_pin(PIN_SDA) << i);
		out(0, 1);
		i2c_delay();
	}
	
	set_output(PIN_SDA);

	switch (ack_control) {
		case SEND_ACK:
			out(0, 0);
			i2c_delay();
			out(1, 0);
			i2c_delay();
			out(0, 1);
			i2c_delay();
			break;
		case SEND_NACK:
			out(0, 1);
			i2c_delay();
			out(1, 1);
			i2c_delay();
			out(0, 1);
			i2c_delay();
			break;
		default:
			fprintf(stderr, "error: unknown ack control mode!\n");
			i2c_stop();
			i2c_cleanup();
			exit(-1);
	}

	return val;
}
