/*
    s_lm75 - read out temperature sensor (LM75) in Mobilcom Surfstations
             (Mainboard: 1030N).
    
    Thomas Fischl <tfischl@gmx.de> 

    Uses "PCI Library" by Martin Mares.    

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    Last change: 09/22/2004

    LM75 <-> CS5530
    SCL   -  GPIO2
    SDA   -  GPIO3
    No pullups to Vcc. Only one 10k between SCL and SDA.
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include "i2c.h"

int main(void) {
	int ret, b1, b2, temp;
	
	i2c_init();
	i2c_start();
	
	
	ret = i2c_sendbyte(0x96);
	if (!ret) {
		fprintf(stderr, "error: LM75 not found!\n");
		i2c_cleanup();
		exit(-1);
	}
	
	ret = i2c_sendbyte(0x00);
	if (!ret) {
		fprintf(stderr, "error: NACK from LM75 while setting regpointer!\n");
		i2c_cleanup();
		exit(-1);
	}

	i2c_stop();
	i2c_start();
	
	ret = i2c_sendbyte(0x97);
	if (!ret) {
		fprintf(stderr, "error: NACK from LM75 while requesting data!\n");
		i2c_cleanup();
		exit(-1);
	}

	b1 = i2c_readbyte(SEND_ACK);
	b2 = i2c_readbyte(SEND_NACK);
	i2c_stop();

	temp = (b1 << 1) | (b2 >> 7);
	if ((temp & (1<<8)) == (1<<8)) {
		temp = -(~(temp - 1) & 0x1FF);
	}
	
	printf("%.1f\n", (double) temp / 2);
	
	i2c_cleanup();

	return 0;
}
